rm (list = ls())

library(timelineS)
library(lubridate)
library(ggplot2)
library(dplyr)
library(reshape2)
library(rPref)
library(mco)
library(MASS)
library(scales)
library(parallel) 

plot.timeline <- function(lifetime, events.name, start.Date, plot.Name) {
  dataP <- data.frame(Events = events.name, Event_Dates = ymd(start.Date) +
                        years(lifetime))
  timelineS(dataP, main = plot.Name,
            labels = events.name, label.direction = "up", label.position = 3)
}

dist.Events <- function (lifetime, events, start.Date, option.Name) {
  #sort the events
  events <- sort(events, decreasing = T)
  #create the distribution of the events
  distribution.events <- sapply(events, seq, from = 0, to = lifetime)
  #all events unlisted
  all.events <- melt(distribution.events)
  colnames(all.events) <- c("frequency", "event")
  #sort the events ascending
  all.events <- all.events[order(all.events$frequency),]
  #get the unique sequence of events
  unique.events <- all.events[!duplicated(all.events$frequency),]
  unique.events$event[which(unique.events$frequency == 0)] <- "DC"
  #plot the timeline
  plot.timeline(unique.events$frequency, unique.events$event, start.Date, option.Name) 
  return(unique.events)
}

combine.lifeTimelines <- function(product.1, product.2, p1.dur, p2.dur) {
  #function body
  
  
  base <- list(Names = c(), frequency = c(), duration = c())
  base$frequency <- sort(unique(c(product.1$frequency, product.2$frequency)), decreasing = FALSE)
  base$Names[1] <- "DC"
  base$duration[1] <- 0
  base$Names[length(base$frequency)] <- "END"
  base$duration[length(base$frequency)] <- 0
  
  for(index in 2:(length(base$frequency) - 1)) {
    phase.1 <- product.1$event[which(product.1$frequency == base$frequency[index])] 
    phase.2 <- product.2$event[which(product.2$frequency == base$frequency[index])] 
    if(length(phase.1) != 0 && length(phase.2) != 0) { base$Names[index] <- paste(phase.1, phase.2)
    base$duration[index] <- max(p1.dur[which(names(p1.dur) == phase.1)],
                                p2.dur[which(names(p2.dur) == phase.2)])
    } else if(length(phase.1) != 0 && length(phase.2) == 0) {
      base$Names[index] <- phase.1
      base$duration[index] <- p1.dur[which(names(p1.dur) == phase.1)]
    } else if (length(phase.1) == 0 && length(phase.2) != 0) {
      base$Names[index] <- phase.2
      base$duration[index] <- p2.dur[which(names(p2.dur) == phase.2)]
    } else {
      base$duration[index] <- 0
    }
  }
  return(base)
  
}

combine.multiple.lifeTimelines <- function(products.list, durations.list) {
  # Initialize the combined product with the first product
  combined.product <- products.list[[1]]
  combined.duration <- durations.list[[1]]
  
  # Iteratively combine each additional product
  for (i in 2:length(products.list)) {
    combined.product <- combine.lifeTimelines(combined.product, products.list[[i]], 
                                              combined.duration, durations.list[[i]])
    # Update the combined durations with the maximum of both
    combined.duration <- mapply(max, combined.product$duration, durations.list[[i]], 
                                SIMPLIFY = FALSE, USE.NAMES = TRUE)
  }
  
  return(combined.product)
}

par(mfrow = c(2, 1))
start.Date = "2020-01-01"
lifetime <- 70

#---------------------first maintenance option-------------------------------------------------

# Updated events according to the new maintenance schedule with specific event names
events.Op1.Road <- c(INSP.r = 3, RPR.r = 8, RPT.r = 15, END.r = lifetime)
events.Op1.OfficeBuilding <- c(INSP.o = 10, RPR.o = NA, RPT.o = 60, END.o = lifetime) # NA for Event 3 as it's not applicable
events.Op1.ResidentialBuilding <- c(INSP.res = 7, RPR.res = 30, RPT.res = 70, END.res = lifetime)
events.Op1.MultipurposeBuilding <- c(INSP.mp = 1, RPR.mp = 6, RPT.mp = 13, END.mp = lifetime)
events.Op1.MultiStoryBuilding <- c(INSP.ms = 3, RPR.ms = 10, RPT.ms = 18, END.ms = lifetime)
events.Op1.Rail <- c(INSP.rl = 3, RPR.rl = 7, RPT.rl = 12, END.rl = lifetime)

# Define updated durations for these events according to the provided maintenance schedule and previous table.
duration.ev.Road <- c(INSP.r = 2, RPR.r = 5, RPT.r = 14) # Assuming the same duration as rail from the previous table
duration.ev.OfficeBuilding <- c(INSP.o = 3, RPR.o = NA, RPT.o = 30) # No Event 3, assuming 2 days for overhaul from 'Event 2' of GLT-Concrete Slab
duration.ev.ResidentialBuilding <- c(INSP.res = 4, RPR.res = 14, RPT.res = 30) # Assuming durations similar to External Wall LSC
duration.ev.MultipurposeBuilding <- c(INSP.mp = 1, RPR.mp = 3, RPT.mp = 10) # Assuming 2 days for maintenance from GLT-Concrete Slab
duration.ev.MultiStoryBuilding <- c(INSP.ms = 2, RPR.ms = 7, RPT.ms = 20) # Assuming durations similar to Multi-Story Building from the previous table
duration.ev.Rail <- c(INSP.rl = 1, RPR.rl = 3, RPT.rl = 17) # Assuming the same duration as rail from the previous table

# Define design options for these types of buildings and infrastructure
design.Options.MultipurposeBuilding <- list(design.Op1.MultipurposeBuilding <- "Multipurpose Building Design Option")
design.Options.OfficeBuilding <- list(design.Op1.OfficeBuilding <- "Office Building Design Option")
design.Options.MultiStoryBuilding <- list(design.Op1.MultiStoryBuilding <- "Multi-Story Building Design Option")
design.Options.ResidentialBuilding <- list(design.Op1.ResidentialBuilding <- "Residential Building Design Option")
design.Options.Rail <- list(design.Op1.Rail <- "Rail")
design.Options.Road <- list(design.Op1.Road <- "Road")

# Call a function to distribute events over a lifetime for maintenance
maintenance.MultipurposeBuilding <- dist.Events(lifetime, events.Op1.MultipurposeBuilding, start.Date, design.Options.MultipurposeBuilding)
maintenance.OfficeBuilding <- dist.Events(lifetime, events.Op1.OfficeBuilding, start.Date, design.Options.OfficeBuilding)
maintenance.MultiStoryBuilding <- dist.Events(lifetime, events.Op1.MultiStoryBuilding, start.Date, design.Options.MultiStoryBuilding)
maintenance.ResidentialBuilding <- dist.Events(lifetime, events.Op1.ResidentialBuilding, start.Date, design.Options.ResidentialBuilding)
maintenance.Rail <- dist.Events(lifetime, events.Op1.Rail, start.Date, design.Options.Rail)
maintenance.Road <- dist.Events(lifetime, events.Op1.Road, start.Date, design.Options.Road)

# Now you need to create a list of all products and their corresponding durations
# For example:
products.list <- list(maintenance.MultipurposeBuilding, maintenance.OfficeBuilding, maintenance.MultiStoryBuilding, maintenance.ResidentialBuilding, maintenance.Rail, maintenance.Road)
durations.list <- list(duration.ev.MultipurposeBuilding, duration.ev.OfficeBuilding, duration.ev.MultiStoryBuilding, duration.ev.ResidentialBuilding, duration.ev.Rail, duration.ev.Road)

# Call the function to combine all product lifetimes
integrated.interv <- combine.multiple.lifeTimelines(products.list, durations.list)



sum(integrated.interv$duration)


#------------------------------second maintenance option-------------------------------------------------

# Updated events according to the new maintenance schedule with specific event names
events.Op1.Road <- c(INSP.r = 3, RPR.r = 8, RPT.r = 15, END.r = lifetime)
events.Op1.OfficeBuilding <- c(INSP.o = 10, RPR.o = NA, RPT.o = 60, END.o = lifetime) # NA for Event 3 as it's not applicable
events.Op1.ResidentialBuilding <- c(INSP.res = 7, RPR.res = 30, RPT.res = 70, END.res = lifetime)
events.Op1.MultipurposeBuilding <- c(INSP.mp = 1, RPR.mp = 6, RPT.mp = 13, END.mp = lifetime)
events.Op1.MultiStoryBuilding <- c(INSP.ms = 5, RPR.ms = 10, RPT.ms = 10, END.ms = lifetime)
events.Op1.Rail <- c(INSP.rl = 3, RPR.rl = 7, RPT.rl = 10, END.rl = lifetime)

# Define updated durations for these events according to the provided maintenance schedule and previous table.
duration.ev.Road <- c(INSP.r = 2, RPR.r = 5, RPT.r = 14) # Assuming the same duration as rail from the previous table
duration.ev.OfficeBuilding <- c(INSP.o = 3, RPR.o = NA, RPT.o = 30) # No Event 3, assuming 2 days for overhaul from 'Event 2' of GLT-Concrete Slab
duration.ev.ResidentialBuilding <- c(INSP.res = 4, RPR.res = 14, RPT.res = 30) # Assuming durations similar to External Wall LSC
duration.ev.MultipurposeBuilding <- c(INSP.mp = 1, RPR.mp = 3, RPT.mp = 10) # Assuming 2 days for maintenance from GLT-Concrete Slab
duration.ev.MultiStoryBuilding <- c(INSP.ms = 2, RPR.ms = 7, RPT.ms = 20) # Assuming durations similar to Multi-Story Building from the previous table
duration.ev.Rail <- c(INSP.rl = 1, RPR.rl = 3, RPT.rl = 17) # Assuming the same duration as rail from the previous table

# Define design options for these types of buildings and infrastructure
design.Options.MultipurposeBuilding <- list(design.Op1.MultipurposeBuilding <- "Multipurpose Building Design Option")
design.Options.OfficeBuilding <- list(design.Op1.OfficeBuilding <- "Office Building Design Option")
design.Options.MultiStoryBuilding <- list(design.Op1.MultiStoryBuilding <- "Multi-Story Building Design Option")
design.Options.ResidentialBuilding <- list(design.Op1.ResidentialBuilding <- "Residential Building Design Option")
design.Options.Rail <- list(design.Op1.Rail <- "Rail")
design.Options.Road <- list(design.Op1.Road <- "Road")

# Call a function to distribute events over a lifetime for maintenance
maintenance.MultipurposeBuilding <- dist.Events(lifetime, events.Op1.MultipurposeBuilding, start.Date, design.Options.MultipurposeBuilding)
maintenance.OfficeBuilding <- dist.Events(lifetime, events.Op1.OfficeBuilding, start.Date, design.Options.OfficeBuilding)
maintenance.MultiStoryBuilding <- dist.Events(lifetime, events.Op1.MultiStoryBuilding, start.Date, design.Options.MultiStoryBuilding)
maintenance.ResidentialBuilding <- dist.Events(lifetime, events.Op1.ResidentialBuilding, start.Date, design.Options.ResidentialBuilding)
maintenance.Rail <- dist.Events(lifetime, events.Op1.Rail, start.Date, design.Options.Rail)
maintenance.Road <- dist.Events(lifetime, events.Op1.Road, start.Date, design.Options.Road)

# Now you need to create a list of all products and their corresponding durations
# For example:
products.list <- list(maintenance.MultipurposeBuilding, maintenance.OfficeBuilding, maintenance.MultiStoryBuilding, maintenance.ResidentialBuilding, maintenance.Rail, maintenance.Road)
durations.list <- list(duration.ev.MultipurposeBuilding, duration.ev.OfficeBuilding, duration.ev.MultiStoryBuilding, duration.ev.ResidentialBuilding, duration.ev.Rail, duration.ev.Road)

# Call the function to combine all product lifetimes
integrated.interv <- combine.multiple.lifeTimelines(products.list, durations.list)


sum(integrated.interv$duration)


#---------------------exploring predefined design space-------------------------------------------------

design.explore <- function(events1, events2) {
  results <- c()
  for(i in 1: dim(events1)[1]) {
    ev1 <- unlist(events1[i, ])
    dist.1 <- dist.Events(lifetime, ev1, start.Date, design.Options.MultipurposeBuilding)
    dur.ev1 <- ev1/2
    for (j in 1: dim(events1)[1]) {
      ev2 <- unlist(events2[j, ])
      dist.2 <- dist.Events(lifetime, ev2, start.Date, design.Options.Road)
      dur.ev2 <- ev2/2
      combined.lifetime <- combine.lifeTimelines(dist.1, dist.2, dur.ev1, dur.ev2)
      min.dist.int <- min(abs(combined.lifetime$frequency[1:(length(combined.lifetime$frequency) - 1)] -
                                combined.lifetime$frequency[2:length(combined.lifetime$frequency)]))
      results <- rbind(results, c(ev1, ev2,  dur = sum(combined.lifetime$duration), dist.inter = min.dist.int))
    }
  }
  return(as.data.frame(results))
}

n.grid <- 5

# Here we schedule buildings and infrastructrure maintanence together
#For this the systems critical Product is the one with the lowest Servicelife (MP Building 50 years and for Infrastructure its Road 18 Years)

events.grid.MultipurposeBuilding <- expand.grid(INSP.mp = sample(seq(3,7, by = 1), n.grid),
                                                RPR.mp = sample(seq(7,11,1), n.grid),
                                                RPT.mp = sample(seq(21, 25, 1), n.grid))

events.grid.Road <- expand.grid(INSP.r = sample(seq(1, 5, by = 1), n.grid),
                                RPR.r = sample(seq(6,10,1), n.grid),
                                RPT.r = sample(seq(12, 16, 1), n.grid))

response.space <- design.explore(events.grid.MultipurposeBuilding, events.grid.Road)

p <- low(dur)* high(dist.inter)

sky <- psel(response.space, p)

pareto2 <- psel(response.space, p, top = nrow(response.space))

ggplot(response.space, aes(x = dur, y = dist.inter)) +
  geom_point(shape = 21) +
  geom_point(data = pareto2, size = 3, aes(color = factor(pareto2$.level)))

show_front <- function(pref) {
  plot(response.space$dur, response.space$dist.inter)
  sky <- psel(response.space, pref)
  plot_front(response.space, pref, col = rgb(0, 0, 1))
  points(sky$dur, sky$dist.inter, lwd = 3) }

show_front(p)

p <- high(dur) * low(dist.inter)
show_front(p)

#---------------------life cycle inventory and analysis---------------------------

LCI.materials <- read.csv("LCImaterials-noStoneBallast.csv")

#Community Building - RCGLT slab --------------------------------------------------------------
LCA.slab <- function(length, width, diameter, composite.Option, materials, dist.Events) {
  
  materials.split <- split(materials, materials$scope)
  
  #new line to get the interventions 
  s <- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  
  #initialize variables 
  composite.volume <- 0 
  reinforcement.weight <-0
  interventions.composite <- 0
  
  #computing the volume of composite elements (excluding reinforcement)
  #Option 1
  if (composite.Option == "RCGLT") {
    thickness <- 0.154
    composite.volume <- thickness * length * width * 0.5 + 
      ((pi * ((diameter)^2)/4 ) * length) + ((pi * ((diameter)^2)/4 ) * (width*0.5))
    interventions.composite <- s2$INSP.mp + s2$RPT.mp #+ 0.25 * s2$RPR.mp
  }
  
  # Create matrix
  composite <- mutate(materials.split[[composite.Option]], building.Q = composite.volume, interventions = interventions.composite)
  
  LCA.matrix <- rbind(composite)
  
  # Calculate total amount of energy and emissions per material
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * composite.volume,
                       materials.LC = TotalMaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2, 
                       NOX.LC = materials.LC * NOX,
                       SO2.LC = materials.LC * SO2) 
  
  # Calculate total amount of energy and emissions for the design options
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOX = sum(LCA.matrix$NOX.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  
  # Return results
  return(LCA.results)
}

# Initialize variables
b.length <- 7 # units in m
b.width <- 1.2 # units in m
bd.diameter <- 0.06 #units in m

# Vector creation
composite.Option <- c("RCGLT")

# Extract the information specific to a design option.
Option.mp <- LCA.slab(b.length, b.width, bd.diameter, composite.Option[1], LCI.materials, maintenance.MultipurposeBuilding)

# Result for all the options
results <- as.data.frame(rbind(Option.mp = unlist(Option.mp)))

print(results)



#Residential Building - LVL beam --------------------------------------------------------------

LCA.beam <- function(length, beam.Option, materials, dist.Events) {
  # function’s body
  lvl.section <- 0.034

  materials.split <- split(LCI.materials, LCI.materials$scope)
  
  #new line to get the interventions 
  s <- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  
  # calculate the volume of the beam based on different materials strategies
  if(beam.Option == "LVL") {
    beam.volume <- length * lvl.section + 0.0015872
    interventions.beam <- s2$INSP.res + s2$RPT.res + 0.25 * s2$RPR.res
  }
  
  beam <- mutate(materials.split[[beam.Option]], beam.Q = beam.volume, 
                 interventions = interventions.beam)
  
  LCA.matrix <- rbind(beam)
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * beam.Q,
                       materials.LC = TotalMaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2,
                       NOX.LC = materials.LC * NOX,
                       SO2.LC = materials.LC * SO2)
  # Debug: Print the head of the LCA matrix to see if it's working
  print(head(LCA.matrix))
  
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOX = sum(LCA.matrix$NOX.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  return(LCA.results)
} 
b.length <- 6 #units: m
beam.Option <- c("LVL")

Option.res <- LCA.beam(b.length, beam.Option[1], LCI.materials, maintenance.ResidentialBuilding)
results <- as.data.frame(rbind(Option.res = unlist(Option.res)))

print(results)



#Road System - Recycled Asphalt Pavement --------------------------------------------------------------
LCA.road <- function(length, width, road.Option, materials, dist.Events) {
  
  materials.split <- split(LCI.materials, LCI.materials$scope)
  
  length <- 0
  width <- 0
  
  #new line to get the interventions 
  s <- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))

if (road.Option == "RAP") { #Recycled Asphalt Pavement
  asphalt_layer <- 0.1
  stabelized_layer <- 0.12
  flexible_subgrade_layer <- 0.13
  road_thickness <- asphalt_layer + stabelized_layer + flexible_subgrade_layer
  road.volume <- road_thickness * road_length * road_width
  interventions.road <- s2$INSP.r + s2$RPT.r + 0.25 * s2$RPR.r
  
}
  road <- mutate(materials.split[[road.Option]], road.Q = road.volume, 
                 interventions = interventions.road)
  
  LCA.matrix <- rbind(road)
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * road.Q,
                       materials.LC = TotalMaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2,
                       NOX.LC = materials.LC * NOX,
                       SO2.LC = materials.LC * SO2)
  
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOX = sum(LCA.matrix$NOX.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  return(LCA.results)
}
road_width <- 6
road_length <- 1000

road.Option <- c("RAP")
Option.r <- LCA.road(road_length, road_width, road.Option[1], LCI.materials, maintenance.Road)
results <- as.data.frame(rbind(Option.r = unlist(Option.r)))

print(results)

LCI.materials <- read.csv("LCImaterials-noStoneBallast.csv")
#Multi-Story Building - LSC Wall ---------------------------------------------------------
LCA.building <- function(length, height, wall.Option, materials, dist.Events) {
  
  materials.split <- split(LCI.materials, LCI.materials$scope)
  
  #new line to get the interventions 
  s <- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  
  # calculate the volume of the wall based on different materials strategies
  if (wall.Option == "LSC") {
    wall.volume <- length * height
    interventions.wall <- s2$INSP.ms + 0.25 * s2$RPR.ms
  }
  
  wall <- mutate(materials.split[[wall.Option]], building.Q = wall.volume, interventions = interventions.wall)
  
  LCA.matrix <- rbind(wall)
  
  # total amount of energy
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * building.Q,
                       materials.LC = TotalMaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2 ,
                       NOX.LC = materials.LC * NOX ,
                       SO2.LC = materials.LC * SO2 )
  
  # total amount of energy for the wall options
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOX = sum(LCA.matrix$NOX.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  
  return(LCA.results)
}

# variables
b.length <- 1  # units: m
b.height <- 1  # units m

# vector
wall.options <- c("LSC")

Option.ms <- LCA.building(b.length, b.height, wall.options[1], LCI.materials, maintenance.MultiStoryBuilding)

results <- as.data.frame(rbind(Option.ms = unlist(Option.ms)))

print(results)



LCI.materials <- read.csv("LCImaterials-noStoneBallast.csv")
#Office Building - HSS column ----------------------------------------------------------
LCA.column <- function(weight, height, column.Option, materials, dist.Events) {
  
  
  materials.split <- split(materials, materials$scope)
  
  circular.steel.section.unitWeight <- 85.2
  high.strength.steel.section.unitWeight <- 83.2
  composite.section.unitWeight <- 270
  column.M <- weight * height
  
    s<- summary(as.factor(dist.Events$event))
    s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  
  # calculate the mass of the column based on different strategies
  if (column.Option == "HSS") {
    column.mass <- high.strength.steel.section.unitWeight * height
    interventions.column <- s2$INSP.o + s2$RPT.o
  } 
  column <- mutate(materials.split[[column.Option]],  column.M = column.mass, interventions = interventions.column)
  LCA.matrix <- rbind(column)
  
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * column.M, 
                       materials.LC = TotalMaterials.Q * interventions, 
                       Energy.LC = materials.LC * energy, 
                       CO2.LC = materials.LC * CO2,
                       NOX.LC = materials.LC * NOX, 
                       SO2.LC = materials.LC * SO2)
  
  
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOX = sum(LCA.matrix$NOX.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  
  return(LCA.results)
  
}

# initialise the variables needed for the function  
c.height <- 5 # units: m 
c.weight <- 85.2 #units kg/m

#vector containing different design options for column
column.options <- c("HSS")

# create options 
Option.o <- LCA.column(c.weight, c.height, column.options[1], LCI.materials, maintenance.OfficeBuilding)

# create a matrix containing the results for all the options
results <- as.data.frame(rbind(Option.o = unlist(Option.o)))

print(results)




LCI.materials <- read.csv("LCImaterials-noStoneBallast.csv")
#Rail Option - RAIL ---------------------------------------------------------------------
LCA.Railbridge <- function(length, width, height, deck.Option, materials, dist.Events) { 
  
  prefab.girder.Section <- 0.78
  
  materials.split <- split(materials, materials$scope)
  
  s<- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  
  if (deck.Option == "RAIL") {
    n <-round(width / 3.75, 0)
    deck.volume <- (0.5 * length * width * height) + (n *prefab.girder.Section * length)
    interventions.deck <- s2$INSP.rl + s2$RPT.rl + 0.25 * s2$RPR.rl
  }
  
  deck <- mutate(materials.split[[deck.Option]], deck.V = deck.volume, interventions = interventions.deck)
  
  LCA.matrix <- rbind(deck)
  
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * deck.V, 
                       materials.LC = TotalMaterials.Q * interventions, 
                       Energy.LC = materials.LC * energy, 
                       CO2.LC = materials.LC * CO2, 
                       NOX.LC = materials.LC * NOX, 
                       SO2.LC = materials.LC * SO2)
  
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC), 
                      CO2 = sum(LCA.matrix$CO2.LC), 
                      NOX = sum(LCA.matrix$NOX.LC), 
                      SO2 = sum(LCA.matrix$SO2.LC))
  
  return(LCA.results)
  
}
b.length <- 16 # units: m 
b.width <- 15 #units m 
bd.height <- 0.25 #units m 


deck.Options <- c("RAIL") 

Option.rl <- LCA.Railbridge(b.length, b.width, bd.height, deck.Options[1], LCI.materials, maintenance.Rail)

results <- as.data.frame(rbind(Option.rl = unlist(Option.rl)))

print(results)

#INTEGRATED EMISSIONS
integrated.Design <- as.data.frame(list(Energy = Option.mp$Energy + Option.res$Energy + Option.o$Energy + Option.ms$Energy + Option.r$Energy + Option.rl$Energy,
                                        CO2 = Option.mp$CO2 + Option.res$CO2 + Option.o$CO2 + Option.ms$CO2 + Option.r$CO2 + Option.rl$CO2,
                                        NOX = Option.mp$NOX + Option.res$NOX + Option.o$NOX + Option.ms$NOX + Option.r$NOX + Option.rl$NOX, 
                                        SO2 = Option.mp$SO2 + Option.res$SO2 + Option.o$SO2 + Option.ms$SO2 + Option.r$SO2 + Option.rl$SO2))
print(integrated.Design)

#barplots
barplot1 <- barplot(as.matrix(integrated.Design[, "Energy", drop = FALSE], ), beside = TRUE, col = c("mistyrose"),
                    names.arg = row.names(integrated.Design), main = "Energy", ylab = "Value (MJ/t)")

barplot2 <- barplot(as.matrix(integrated.Design[, "CO2", drop = FALSE]), beside = TRUE, col = c("lightcyan"),
                    names.arg = row.names(integrated.Design), main = "CO2", ylab = "Value (kg/m3 CO2 eq)")

barplot3 <- barplot(as.matrix(integrated.Design[, "NOX", drop = FALSE]), beside = TRUE, col = c("peachpuff"),
                    names.arg = row.names(integrated.Design), main = "NOx", ylab = "Value (kg/m3 NOx eq)")

barplot4 <- barplot(as.matrix(integrated.Design[, "SO2", drop = FALSE]), beside = TRUE, col = c("lavender"),
                    names.arg = row.names(integrated.Design), main = "SO2", ylab = "Value (kg/m3 SO2 eq)")




Energy.costs <- 0.128
CO2.unitcost <- 26 # per metric tone
NOX.unitCost <- 42 # per metric tone
SO2.unitCosts <- 85 # per metric tone

integrated.Costs <- mutate(integrated.Design, Costs = (Energy * Energy.costs +
                                                          CO2*CO2.unitcost +
                                                          NOX * NOX.unitCost +
                                                          SO2 * SO2.unitCosts)/10^9)
print(integrated.Costs)

barplot(as.matrix(integrated.Costs [, "Costs", drop = FALSE], ), beside = TRUE, col = c("lightblue"),
        names.arg = row.names(integrated.Design), main = "Costs", ylab = "Value (€/m*t)")


#-----------------------------------------------------Multi-Optimization-------------------------------------
fitness <- function(x){
 
  z <- numeric(7)
  x <- round(x, 0)
  y <- expand.grid(INSP.mp = x[1], RPR.mp = x[2], RPT.mp = x[3], INSP.r = x[4], RPR.r = x[5], RPT.r = x[6])
 
  dur.ev1 <- unlist(y[1:3] / 2)
  dur.ev2 <- unlist(y[4:6] / 2)
  
  dist.1 <- apply(y[1:3], 1, function(row) dist.Events(lifetime = lifetime, events = row, start.Date = start.Date, option.Name = "design.Options.MultipurposeBuilding"))
  dist.2 <- apply(y[4:6], 1, function(row) dist.Events(lifetime = lifetime, events = row, start.Date = start.Date, option.Name = "design.Options.Road"))
  
  results <- combine.lifeTimelines(dist.1[[1]], dist.2[[1]], dur.ev1, dur.ev2)
  
  z[1] <- sum(results[["duration"]])
  z[2] <- -min(abs(results$frequency[1:(length(results$frequency) - 1)] - results$frequency[2:length(results$frequency)]))
  
  #---------Parameters--------- 
  #slab
  mp.length <- 7 # units in m
  # mp.width =  1.2 # units in m
  mp.diameter <- 0.06 #units in m
  
  #road
  #road_width <- x[12]
  road_length <- 1000
  optimum_width <- x[12]
  
  Energy.costs <- 0.128 
  CO2.unitcost <- 26 
  NOX.unitCost <- 42 
  SO2.unitCosts <- 85 
  

  product.1.Option.mp <- LCA.slab(mp.length, optimum_width, mp.diameter, composite.Option, LCI.materials, dist.1[[1]])
  
  product.2.Option.r <- LCA.road(road_length, optimum_width, road.Option, LCI.materials, dist.2[[1]])
  
  integrated.system <- as.data.frame(list(Energy = product.1.Option.mp$Energy + product.2.Option.r$Energy, # + product.2.Option.res$Energy + product.3.Option.o$Energy + product.4.Option.ms$Energy + product.6.Option.rl$Energy
                                          CO2 = product.1.Option.mp$CO2 + product.2.Option.r$CO2, # + product.2.Option.res$CO2 + product.3.Option.o$CO2 + product.4.Option.ms$CO2 + product.6.Option.rl$CO2
                                          NOX = product.1.Option.mp$NOX + product.2.Option.r$NOX, #+ product.2.Option.res$NOX + product.3.Option.o$NOX + product.4.Option.ms$NOX + product.6.Option.rl$NOX
                                          SO2 = product.1.Option.mp$SO2 + product.2.Option.r$SO2)) # + product.2.Option.res$SO2 + product.3.Option.o$SO2 + product.4.Option.ms$SO2 + product.6.Option.rl$SO2
  print(integrated.system)
  integrated.system <- mutate(integrated.system,
                              Costs = (Energy * Energy.costs +
                                         CO2*CO2.unitcost +
                                         NOX * NOX.unitCost +
                                         SO2 * SO2.unitCosts)/10^9)
  
  z[3] <- integrated.system$Energy
  z[4] <- integrated.system$CO2
  z[5] <- integrated.system$NOX
  z[6] <- integrated.system$SO2
  z[7] <- integrated.system$Costs
  
  return (z)
  print(z)
}

#increasing population size to 100 and the number of generations to 10
r2 <- nsga2(fitness, idim = 7, odim = 7,
            generations=10, popsize=100,
             lower.bounds=c(3, 7, 21, 1, 6, 12, 1),
            upper.bounds= c(7, 11, 25, 5, 10, 16, 8))


r2$value
r2$par
r2$pareto.optimal
which(r2$pareto.optimal==TRUE)

print(table(r2$pareto.optimal))

#Plotting
r2Results <- as.data.frame(r2$value)
outNames <- c("duration", "interv.dist", "energy", "co2", "nox", "so2", "cost" )
colnames(r2Results) <- outNames


pareto3 <- as.data.frame(paretoFront(r2)) 
colnames(pareto3) <- outNames

ggplot(r2Results, aes(x = duration, y = cost)) + geom_point(shape = 21) +
geom_point(data = pareto3, size = 3, color="red") + geom_line(data = pareto3, color="blue")

input.params <- round(r2$par, 0)

print(head(r2$par))

all.results <- cbind(input.params, r2Results, r2$pareto.optimal)

print(head(input.params))
print(head(r2Results))

colnames(all.results) <- c("INSP.mp", "RPR.mp", "RPT.mp", "INSP.r", "RPR.r", "RPT.r", "optimal.width", "duration", "interv.dist", "energy", "co2", "NOX", "so2", "cost", "pareto")

print(head(all.results))

par(mfrow = c(1,1))

parcoord(all.results[, 1:15], var.label = T,
         col = ifelse (all.results$pareto == TRUE, "indianred", "skyblue2"), 
         lty = ifelse(all.results$pareto == TRUE, 1, 3),
         lwd = ifelse(all.results$pareto == TRUE, 3, 1))


